/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.container.lifecycle;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.jbi.JBIException;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.Service;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly;
import org.ow2.petals.jbi.messaging.routing.RouterService;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.system.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This class manage the life cycle of deployed service assemblies
 * 
 * @author Olivier Fabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 */
@FractalComponent
public class ServiceAssemblyLifeCycle extends LifeCycleAbstract {

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * Platform component :: SystemState Service
     */
    @Requires(name = "systemstate", signature = org.ow2.petals.system.systemstate.SystemStateService.class)
    private SystemStateService systemState;

    /**
     * JBI messaging component :: Router Service
     */
    @Requires(name = "router", signature = org.ow2.petals.jbi.messaging.routing.RouterService.class)
    private RouterService routerService;

    /**
     * The component
     */
    @Service
    private Component comp;

    /**
     * The SA JBI descriptor
     */
    private ServiceAssembly serviceAssembly;

    /**
     * List of SU life-cycles
     */
    private List<ServiceUnitLifeCycle> serviceUnitsLifeCycles;

    public ServiceAssemblyLifeCycle() {
        super();
    }

    /**
     * Accessor to the fractal component
     * 
     * @return the fractal component
     */
    public Component getComponent() {
        return this.comp;
    }

    /**
     * Initialize the life-cycle. This is not the init() step of the life-cycle.
     * 
     * @param mbeanName
     *            name of the MBean for this LifeCycle. In fact, the
     *            DeploymentServiceMBean name is used as there is no
     *            SALifeCycleMBean
     * @param log
     * @throws JBIException
     */
    public void init(ServiceAssembly serviceAssembly) throws JBIException {
        this.serviceAssembly = serviceAssembly;
        this.name = serviceAssembly.getIdentification().getName();

        // set the state to SHUTDOWN, as the JBI specifications specifies
        this.setState(SHUTDOWN);
    }

    /**
     * Do nothing
     */
    @Override
    public void doInit() throws JBIException {
        // Do nothing
    }

    @Override
    public void doShutdown() throws JBIException {
        final List<ServiceUnitLifeCycle> suLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                this.serviceUnitsLifeCycles);
        final List<ServiceUnitLifeCycle> reversedSuLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                suLifeCycles);
        Collections.reverse(reversedSuLifeCycles);

        for (final ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
            try {
                if (STOPPED.equals(suLifeCycle.getCurrentState())) {
                    suLifeCycle.shutDown();
                } else if (!this.forcedState) {
                    throw new JBIException(
                            "Unable to shut down a service unit which is not stopped");
                }
                if (UNKNOWN.equals(suLifeCycle.getCurrentState()) && !this.forcedState) {
                    throw new JBIException("Unable to shutdown a service unit");
                }
            } catch (final JBIException e) {
                if (!this.forcedState) {
                    throw e;
                }
                // continue, ignoring the error
            }
        }

    }

    @Override
    public void doStart() throws JBIException {
        // The JBI specifications force to initialize all the service units
        // before starting all the service units
        for (final ServiceUnitLifeCycle suLifeCycle : this.serviceUnitsLifeCycles) {
            if (SHUTDOWN.equals(suLifeCycle.getCurrentState())) {
                suLifeCycle.init();
            }
            if (UNKNOWN.equals(suLifeCycle.getCurrentState())) {
                throw new JBIException("Unable to initialize a service unit");
            }
        }
        for (final ServiceUnitLifeCycle suLifeCycle : this.serviceUnitsLifeCycles) {
            if (STOPPED.equals(suLifeCycle.getCurrentState())) {
                suLifeCycle.start();
            }
            if (UNKNOWN.equals(suLifeCycle.getCurrentState())) {
                throw new JBIException("Unable to start a service unit");
            }
        }
    }

    @Override
    public void doStop() throws JBIException {
        final List<ServiceUnitLifeCycle> suLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                this.serviceUnitsLifeCycles);
        final List<ServiceUnitLifeCycle> reversedSuLifeCycles = new ArrayList<ServiceUnitLifeCycle>(
                suLifeCycles);
        Collections.reverse(suLifeCycles);
        for (final ServiceUnitLifeCycle suLifeCycle : reversedSuLifeCycles) {
            try {
                if (STARTED.equals(suLifeCycle.getCurrentState())) {
                    suLifeCycle.stop();
                } else if (!this.forcedState) {
                    throw new JBIException("Unable to stop a service unit which is not started");
                }
                if (UNKNOWN.equals(suLifeCycle.getCurrentState()) && !this.forcedState) {
                    throw new JBIException("Unable to stop a service unit");
                }
            } catch (final JBIException e) {
                if (!this.forcedState) {
                    throw e;
                }
            }
        }
    }

    public ServiceAssembly getServiceAssembly() {
        return this.serviceAssembly;
    }

    /**
     * @param serviceAssembly
     *            the serviceAssembly to set
     */
    public void setServiceAssembly(ServiceAssembly serviceAssembly) {
        this.serviceAssembly = serviceAssembly;
    }

    public List<ServiceUnitLifeCycle> getServiceUnitLifeCycles() {
        return this.serviceUnitsLifeCycles;
    }

    /**
     * Register a ServiceUnitLifeCycle into a synchronized map of the
     * ServiceAssemblyLifeCycle
     * 
     * @param suName
     *            name of a service unit
     * @param serviceUnitLifeCycle
     *            life cycle of the service unit
     * @throws JBIException
     */
    public void registerSU(ServiceUnitLifeCycle serviceUnitLifeCycle) throws JBIException {
        if (!this.serviceUnitsLifeCycles.contains(serviceUnitLifeCycle.getSuName())) {
            this.serviceUnitsLifeCycles.add(serviceUnitLifeCycle);
        } else {
            throw new JBIException("Duplicate service unit : same name");
        }
    }

    /**
     * Unregister a ServiceUnitLifeCycle from a synchronized map of the
     * ServiceAssemblyLifeCycle
     * 
     * @param suName
     *            name of a service unit
     */
    public void unregisterSU(ServiceUnitLifeCycle serviceUnitLifeCycle) {
        this.serviceUnitsLifeCycles.remove(serviceUnitLifeCycle);
    }

    @Override
    public void setState(String state) throws JBIException {
        super.setState(state);

        this.routerService.modifiedSALifeCycle(this.serviceUnitsLifeCycles);

        try {
            this.systemState.updateServiceAssemblyState(this.name, state);
        } catch (final Exception e) {
            throw new JBIException("Failed to persist the state of the Service Assembly '"
                    + this.name + "'", e);
        }
    }

    /**
     * 
     * @throws PetalsException
     * @throws JBIException
     * 
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void startComponentLifeCycle() {
        this.log = new LoggingUtil(this.logger);
        this.log.call();

        this.serviceUnitsLifeCycles = new ArrayList<ServiceUnitLifeCycle>();
    }

    /**
     * Stop of the petals component
     * 
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stopComponentLifeCycle() throws JBIException {
        this.log.call();

        if (STARTED.equals(this.getCurrentState())) {
            try {
                this.doStop();
                this.doShutdown();
            } catch (final JBIException e) {
                this.log.error("Failed to stop the Service Assembly '" + this.name + "'", e);
            }
        }

        if (STOPPED.equals(this.getCurrentState())) {
            try {
                this.doShutdown();
            } catch (final JBIException e) {
                this.log.error("Failed to shut down the Service Assembly '" + this.name + "'", e);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.LifeCycleAbstract#getJBIArtifactType()
     */
    @Override
    protected String getJBIArtifactType() {
        return "Service Assembly";
    }
}
