/**
 * PETALS - PETALS Services Platform. Copyright (c) 2005 Fossil E-Commerce,
 * http://www.fossilec.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: InstallationServiceMBean.java 154 2006-03-27 15:30:10Z alouis $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.installation;

import javax.jbi.JBIException;

import org.ow2.petals.container.lifecycle.Installer;
import org.ow2.petals.container.lifecycle.LifeCycleAbstract;

/**
 * This custom interface add methods used by reinstallation tasks of the
 * SystemStateService service.
 * 
 * @author alouis
 * 
 */
public interface InstallationServiceMBean extends javax.jbi.management.InstallationServiceMBean {

    /**
     * Uninstall a previously installed shared library with the given version.
     * 
     * @param slName
     *            the name of the shared name space to uninstall; must be
     *            non-null and non-empty
     * @param version
     *            the Version of the shared name space to uninstall; must be
     *            non-null and non-empty
     * @return true if the uninstall was successful
     */
    public boolean uninstallSharedLibrary(String slName, String version);

    /**
     * Try to unload installer of a component. If the component is in the
     * STARTED state it's stopped and shutdown, if the component is in the
     * STOPPED state it's shutdown. If the installer is in the INSTALLED state,
     * it's uninstalled.
     * 
     * @param compName
     *            the component unique name
     * @return true if the component installer is correctly unloaded, false
     *         otherwise
     */
    boolean forceUnloadInstaller(String compName);

    /**
     * Get the list of components that use the shared library
     * 
     * @param slName
     *            the name of the shared library
     * @return the list of components that uses this shared library
     * @throws Exception if the shared library is not installed
     */
    String[] getInstalledComponentsForSharedLibrary(String slName) throws Exception;

    /**
     * Get the list of all installed shared libraries
     * 
     * @return the list of all installed shared libraries
     */
    String[] getInstalledSharedLibraries();

    /**
     * Try to shutdown all components. If a component can't be shutdown, the
     * shutdowning process isn't aborted. It returns the list of all shutdown
     * components.
     * 
     * @return an array of succesfuly shutdown component names.
     */
    String[] shutdownAllComponents();

    /**
     * Shutdown the component. This puts the component into the
     * {@link LifeCycleAbstract#SHUTDOWN} state.
     * 
     * @param componentName
     *            name of the component to be shutdown; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly shut down
     * @throws JBIException
     *             if the component fails to shut down
     */
    boolean shutdownComponent(String componentName) throws JBIException;

    /**
     * Try to start all components. If a component can't be started, the
     * starting process isn't aborted. It returns the list of all started
     * components.
     * 
     * @return an array of succesfuly started component names.
     */
    String[] startAllComponents();

    /**
     * Start the component. This puts the component into the
     * {@link LifeCycleAbstract#STARTED} state.
     * 
     * @param componentName
     *            name of the component to be started; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly started
     * @throws JBIException
     *             if the component fails to start
     */
    boolean startComponent(String componentName) throws JBIException;

    /**
     * Try to stop all components. If a component can't be stoped, the stoping
     * process isn't aborted. It returns the list of all stoped components.
     * 
     * @return an array of succesfuly stoped component names.
     */
    String[] stopAllComponents();

    /**
     * Stop the component. This puts the component into the
     * {@link LifeCycleAbstract#STOPPED} state.
     * 
     * @param componentName
     *            name of the component to be stopped; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly stopped
     * @throws JBIException
     *             if the component fails to stop
     */
    boolean stopComponent(String componentName) throws JBIException;
    
    /**
     * Try to uninstall all components. If a component can't be uninstalled, the
     * uninstalling process isn't aborted. It returns the list of all
     * uninstalled components.
     * 
     * @return an array of succesfuly uninstalled component names.
     */
    String[] uninstallAllComponents();

    /**
     * Try to uninstall all shared libraries. If a shared library can't be
     * uninstalled, the uninstalling process isn't aborted. It returns the list
     * of all uninstalled shared libraries.
     * 
     * @return an array of succesfuly uninstalled shared library names.
     */
    String[] uninstallAllSharedLibraries();

    /**
     * Uninstall the component. This puts the component into the
     * {@link Installer#UNINSTALLED} state.
     * 
     * @param componentName
     *            name of the component to be uninstalled; must be non-null and
     *            non-empty
     * @return {@code true} if the component is correctly uninstalled
     * @throws JBIException
     *             if the component fails to uninstall
     */
    boolean uninstallComponent(String componentName) throws JBIException;

    /**
     * Try to uninstall all components. If the boolean is set to true, the
     * uninstallation process is forced as this : if the component is in the
     * STARTED state it's stopped and shutdown, if the component is in the
     * STOPPED state it's shutdown. If the installer is in the INSTALLED state,
     * it's uninstalled. If a component can't be uninstalled, the uninstalling
     * process isn't aborted. It returns the list of all uninstalled components.
     * 
     * @return an array of succesfuly uninstalled component names.
     */
    String[] unloadAllInstallers(boolean force);

}
