/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.io.IOException;
import java.net.URI;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.ManagementException;
import org.ow2.petals.system.repository.RepositoryService;
import org.ow2.petals.system.repository.exception.RepositoryException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task copy unzipped entity (component, shared lib, service assembly)
 * package into the repository <br/>
 * Inputs : {@link Jbi}, unzipped package root {@link File} <br/>
 * Outputs : entity install root {@link URI} <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public abstract class AbstractCopyPackageToRepositoryTask extends AbstractLoggableTask {

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositoryService = null;

    public AbstractCopyPackageToRepositoryTask(LoggingUtil log, RepositoryService repositoryService) {
        super(log);
        this.repositoryService = repositoryService;
    }

    public void execute(Context context) throws Exception {
        String entityName = context.getEntityName();
        String entityVersion = context.getEntityVersion();
        File packageFile = context.getUnzipRoot();
        File installUrl = this.copyPackage(entityName, entityVersion, packageFile);

        context.setInstallUrl(installUrl.toURI().toURL());
    }

    public void undo(Context context) throws Exception {
        String entityName = context.getEntityName();
        String entityVersion = context.getEntityVersion();

        if (entityName != null) {
            try {
                this.removeEntityPackage(entityName, entityVersion);
            } catch (IOException e) {
                this.log.error("Unable to remove the entity package", e);
            }
        }
    }

    /**
     * Expand the given package into the system repository
     * 
     * @param descriptor
     *            jbi descriptor of the entity
     * @param packageFile
     *            the entity package root file
     * @return return installation URI of the expanded package, null if it fails
     * @throws ManagementException
     *             if the package copy fails or if returned install root is null
     */
    protected File copyPackage(String entityName, String entityVersion, File packageFile)
            throws ManagementException {
        File installationRoot = null;

        try {
            installationRoot = this.copyEntityPackage(entityName, entityVersion, packageFile);
        } catch (IOException pe) {
            throw new ManagementException(pe);
        } catch (final RepositoryException e) {
            throw new ManagementException(e);
        }
        return installationRoot;
    }

    protected abstract File copyEntityPackage(String entityName, String entityVersion,
            File packageFile) throws IOException, RepositoryException;

    protected abstract void removeEntityPackage(String entityName, String entityVersion)
            throws IOException;

}
