/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: PackageHandler.java 154 2 mai 2006 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.zip.ZipFile;

import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.system.repository.RepositoryService;
import org.ow2.petals.system.repository.RepositoryServiceImpl.EntityType;
import org.ow2.petals.util.ZipUtil;

import static org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE;

/**
 * Utility class for handling packages and package URLs
 * 
 * @author ofabre - EBM Websourcing
 * @author rnaudin - EBM Websourcing
 */
public class PackageHelper {

    /**
     * Load the JBI descriptor from the repository folder of the given entity
     * (component, shared libraries or service assembly).
     * 
     * @param entityName
     *            name of entity
     * @param entityType
     *            type of entity (component, shared libraries or service
     *            assembly)
     * @return the {@link Jbi} object
     * @throws PetalsException
     * @throws PetalsException
     * @throws IOException
     * @throws JBIDescriptorException
     * @throws MalformedURLException
     */
    public static final Jbi loadDescriptor(String entityName, EntityType entityType,
            RepositoryService repositoryService) throws PetalsException, MalformedURLException,
            JBIDescriptorException, IOException {
        Jbi result = null;
        File tmpJBIFile = null;
        InputStream tmpJBIFileAsInputStream = null;

        try {
            switch (entityType) {
                case COMPONENT_TYPE:
                    tmpJBIFile = new File(repositoryService.getComponentInstallDirectory(entityName),
                            JBI_DESCRIPTOR_RESOURCE);
                    break;
                case SA_TYPE:
                    tmpJBIFile = new File(repositoryService.getServiceAssemblyInstallDirectory(entityName), JBI_DESCRIPTOR_RESOURCE);
                    break;
                case SL_TYPE:
                    tmpJBIFile = new File(repositoryService.getSharedLibraryInstallDirectory(entityName), JBI_DESCRIPTOR_RESOURCE);
                    break;
            }

            if (!tmpJBIFile.exists()) {
                throw new PetalsException(
                        "JBI descriptor entry was not found in repository for JBI artefact '" + entityName + "'");
            }

            tmpJBIFileAsInputStream = new FileInputStream(tmpJBIFile);
            result = JBIDescriptorBuilder.buildJavaJBIDescriptor(tmpJBIFileAsInputStream);

        } finally {
            if (tmpJBIFileAsInputStream != null) {
                tmpJBIFileAsInputStream.close();
            }
        }

        return result;
    }

    /**
     * Load the JBI descriptor from the given archive
     * 
     * @param archiveURI
     *            the archive URI
     * @return the {@link Jbi} object
     * @throws IOException
     * @throws JBIDescriptorException
     * @throws MalformedURLException
     * @throws PetalsException
     * @throws PetalsException
     * @throws PetalsException
     */
    public static final Jbi loadDescriptor(URL archiveURL) throws MalformedURLException,
            JBIDescriptorException, IOException, PetalsException {
        Jbi result = null;
        ZipFile zipArchive = null;
        File tmpJBIFile = null;
        InputStream tmpJBIFileAsInputStream = null;

        try {
            zipArchive = new ZipFile(archiveURL.getFile());
            tmpJBIFile = ZipUtil.getEntryAsTemp(zipArchive, JBI_DESCRIPTOR_RESOURCE);

            tmpJBIFileAsInputStream = tmpJBIFile.toURI().toURL().openStream();
            result = JBIDescriptorBuilder.buildJavaJBIDescriptor(tmpJBIFileAsInputStream);
        } finally {
            if (tmpJBIFileAsInputStream != null) {
                tmpJBIFileAsInputStream.close();
            }

            if (zipArchive != null) {
                zipArchive.close();
            }
            
            // delete temporary file
            if (tmpJBIFile != null) {
                tmpJBIFile.delete();
            }
        }

        return result;
    }

    /**
     * Load the JBI descriptor from the unzipped package folder of the given
     * entity (component, shared lib or service assembly).
     * 
     * @param unzipRoot
     *            unzipped package root in work directory
     * @return the {@link Jbi} object
     * @throws PetalsException
     * @throws PetalsException
     * @throws JBIDescriptorException
     * @throws FileNotFoundException
     */
    public static final Jbi loadDescriptor(File packageFile) throws PetalsException,
            FileNotFoundException, JBIDescriptorException {
        Jbi result = null;
        File tmpJBIFile = null;
        InputStream tmpJBIFileAsInputStream = null;

        try {
            tmpJBIFile = new File(packageFile, JBI_DESCRIPTOR_RESOURCE);

            if (!tmpJBIFile.exists()) {
                throw new PetalsException("JBI descriptor not found in the package '" + packageFile.getAbsolutePath() + "'");
            }

            tmpJBIFileAsInputStream = new FileInputStream(tmpJBIFile);

            result = JBIDescriptorBuilder.buildJavaJBIDescriptor(tmpJBIFileAsInputStream);
        } finally {
            try {
                if (tmpJBIFileAsInputStream != null) {
                    tmpJBIFileAsInputStream.close();
                }
            } catch (IOException ex) {
                //Eat the exception (to avoid impacting everyone's code)...bad practice anyway
            }
        }

        return result;
    }

}
