/**
 * PETALS: PETALS Services Platform Copyright (C) 2006 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): Adrien LOUIS
 * --------------------------------------------------------------------------
 * $Id: MessageExchangeDecorator.java,v 1.0 2006/04/11 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.exchange;

import java.io.IOException;
import java.net.URI;
import java.sql.SQLException;
import java.util.Map;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;

import org.ow2.petals.system.persistence.PersistenceService;

/**
 * 
 * This class decorates a real <code>MessageExchange</code>. <br>
 * Each actor (consumer or provider) owns an instance of this class. <br>
 * The decorator checks that the methods which are called by the related
 * component can be used at the current exchange state.
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 */
public class MessageExchangeDecorator implements MessageExchange {

    /**
     * The unique exchange Id of this decorator.
     */
    private String exchangeId;

    /**
     * The decorated Message Exchange. This instance can change during the life
     * of a dectorator instance.
     */
    private MessageExchangeImpl messageExchange;

    /**
     * Role of the observer (consumer or provider). Used for checking that the
     * ME is in the good role before calling a method.
     */
    private Role observerRole;

    /**
     * A flag to know if the exchange is in a send timeout state.
     */
    private boolean timeout;

    private PersistenceService persistenceService;

    /**
     * Create a MessageExchangeDecorator. Set this object as the
     * consumerMEDecorator or providerMEDecorator of the MessageExchangeImpl,
     * depending on the given Role.
     * 
     * @param consumerDeliveryChannel
     */
    public MessageExchangeDecorator(org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl me,
            Role observerRole) {
        this.messageExchange = me;
        this.exchangeId = this.messageExchange.getExchangeId();
        this.observerRole = observerRole;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#createFault()
     */
    public Fault createFault() throws MessagingException {
        this.checkComponentOwnsExchange();
        this.checkExchange();

        return this.messageExchange.createFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#createMessage()
     */
    public NormalizedMessage createMessage() throws MessagingException {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.createMessage();
    }

    public org.ow2.petals.jbi.messaging.endpoint.ServiceEndpoint getConsumerEndpoint() {
        this.checkExchange();

        return this.messageExchange.getConsumerEndpoint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getEndpoint()
     */
    public ServiceEndpoint getEndpoint() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getEndpoint();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getError()
     */
    public Exception getError() {
        this.checkExchange();
        checkComponentOwnsExchange();

        return this.messageExchange.getError();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getExchangeId()
     */
    public String getExchangeId() {
        return this.exchangeId;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getFault()
     */
    public Fault getFault() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getFault();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getInterfaceName()
     */
    public QName getInterfaceName() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getInterfaceName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#getMessage(java.lang.String)
     */
    public NormalizedMessage getMessage(String name) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getMessage(name);
    }

    /**
     * Return the real implementation of this MessageExchange.
     * 
     * @return MessageExchangeImpl
     */
    public MessageExchangeImpl getMessageExchange() {
        this.checkExchange();

        return this.messageExchange;
    }

    /**
     * Get all the normalized messages
     * 
     * @return
     */
    public Map<String, NormalizedMessage> getMessages() {
        this.checkExchange();

        return this.messageExchange.getMessages();
    }

    /**
     * Return the Role that the owner of this object has in the exchange.
     * 
     * @return
     */
    public Role getObserverRole() {
        return this.observerRole;
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getOperation()
     */
    public QName getOperation() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getOperation();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getPattern()
     */
    public URI getPattern() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getPattern();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getProperty(java.lang.String)
     */
    public Object getProperty(String name) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getProperty(name);
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getPropertyNames()
     */
    public Set<?> getPropertyNames() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getPropertyNames();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getRole()
     */
    public Role getRole() {
        this.checkExchange();

        return this.messageExchange.getRole();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getService()
     */
    public QName getService() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.getService();
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#getStatus()
     */
    public javax.jbi.messaging.ExchangeStatus getStatus() {
        this.checkExchange();

        return this.messageExchange.getStatus();
    }

    public boolean isMessageExchangeStored() {
        if (this.messageExchange == null) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Is terminated?
     * 
     * @return
     */
    public boolean isTerminated() {
        this.checkExchange();

        return this.messageExchange.isTerminated();
    }

    /**
     * Is send timeout?
     * 
     * @return
     */
    public boolean isTimeout() {
        return timeout;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#isTransacted()
     */
    public boolean isTransacted() {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        return this.messageExchange.isTransacted();
    }

    /**
     * Persist the decorated exchange. <br>
     * Hold the PersistenceService reference to retrieve the exchange from the
     * persistence service when necessary.
     * 
     * @param persistenceService
     * @throws SQLException
     * @throws IOException
     */
    public void persistExchange(PersistenceService persistenceService) throws SQLException,
            IOException {
        if (this.messageExchange.isPersisted() || this.messageExchange.isMonitored()) {
            persistenceService.updateMessageExchange(this.messageExchange, true, null);
        } else {
            persistenceService.addMessageExchange(this.messageExchange, true, null);
        }
        this.messageExchange = null;
        this.persistenceService = persistenceService;
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#setEndpoint(java.lang.Exception)
     */
    public void setEndpoint(ServiceEndpoint endpoint) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setEndpoint(endpoint);
    }

    /*
     * @see javax.jbi.messaging.MessageExchange#setError(java.lang.Exception)
     */
    public void setError(Exception error) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setError(error);
    }

    /*
     * @see
     * javax.jbi.messaging.MessageExchange#setFault(javax.jbi.messaging.Fault)
     */
    public void setFault(Fault fault) throws MessagingException {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setFault(fault);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.MessageExchange#setInterfaceName(javax.xml.namespace
     * .QName)
     */
    public void setInterfaceName(QName interfaceName) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setInterfaceName(interfaceName);
    }

    /*
     * @seejavax.jbi.messaging.MessageExchange#setMessage(javax.jbi.messaging.
     * NormalizedMessage, java.lang.String)
     */
    public void setMessage(NormalizedMessage msg, String name) throws MessagingException {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setMessage(msg, name);
    }

    /**
     * Reset a Message Exchange instance to the decorator
     * 
     * @param messageExchange
     */
    public void setMessageExchange(
            org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl messageExchange)
            throws MessagingException {
        if (!this.exchangeId.equals(messageExchange.getExchangeId())) {
            throw new MessagingException("Wrong Message Exchange Id for decorator. Expected Id '"
                    + this.exchangeId + "' and get '" + messageExchange.getExchangeId() + "'");
        }
        this.messageExchange = messageExchange;
    }

    /**
     * Set the observer role
     * 
     * @return
     */
    public void setObserverRole(Role role) {
        this.observerRole = role;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.MessageExchange#setOperation(javax.xml.namespace.
     * QName)
     */
    public void setOperation(QName name) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setOperation(name);
    }

    /**
     * Set the persisted flag.
     * 
     * @param role
     */
    public void setPersisted(boolean persisted) {
        this.checkExchange();

        this.messageExchange.setPersisted(persisted);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.messaging.MessageExchange#setProperty(java.lang.String,
     * java.lang.Object)
     */
    public void setProperty(String name, Object obj) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setProperty(name, obj);
    }

    /**
     * Set the role.
     * 
     * @param role
     */
    public void setRole(Role role) {
        this.checkExchange();

        this.messageExchange.setRole(role);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.messaging.MessageExchange#setService(javax.xml.namespace.QName)
     */
    public void setService(QName service) {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setService(service);
    }

    /*
     * (non-Javadoc)
     * 
     * @seejavax.jbi.messaging.MessageExchange#setStatus(javax.jbi.messaging.
     * ExchangeStatus)
     */
    public void setStatus(ExchangeStatus status) throws MessagingException {
        this.checkExchange();
        this.checkComponentOwnsExchange();

        this.messageExchange.setStatus(status);
    }

    /**
     * Set the terminated flag.
     * 
     * @param terminated
     */
    public void setTerminated(boolean terminated) {
        this.checkExchange();

        this.messageExchange.setTerminated(terminated);
    }

    /**
     * Set the send timeout on the decorator.
     * 
     * @return
     */
    public void setTimeout(boolean timeout) {
        this.timeout = timeout;
    }

    /**
     * Set the transacted flag.
     * 
     * @param terminated
     */
    public void setTransacted(boolean transacted) {
        this.checkExchange();

        this.messageExchange.setTerminated(transacted);
    }

    /**
     * Check that the component owns the message exchange. (Check that the role
     * of the observer is the same than the role of the message exchange)
     * 
     * @throws IllegalStateException
     */
    private void checkComponentOwnsExchange() throws IllegalStateException {
        if (this.observerRole != null && !this.messageExchange.getRole().equals(this.observerRole)) {
            if (this.messageExchange.isTerminated()) {
                throw new IllegalStateException("The Message Exchange has been terminated "
                        + "by another component");
            } else {
                throw new IllegalStateException("The component does not owns the MessageExchange");
            }
        }
    }

    /**
     * Check if the exchange is in memory. if not, retrieve it from the
     * persistence manager
     * 
     * @throws SQLException
     */
    private void checkExchange() throws RuntimeException {
        if (this.messageExchange == null) {
            try {
                this.messageExchange = this.persistenceService.getMessageExchange(exchangeId);
            } catch (SQLException e) {
                throw new RuntimeException("Failed to retrieve Message Exchange with id '"
                        + exchangeId + "' from the persistence service", e);
            } catch (IOException e) {
                throw new RuntimeException("Failed to retrieve Message Exchange with id '"
                        + exchangeId + "' from the persistence service", e);
            } catch (ClassNotFoundException e) {
                throw new RuntimeException("Failed to retrieve Message Exchange with id '"
                        + exchangeId + "' from the persistence service", e);
            }
        }
    }

}
