/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: Router.java,v 1.2 2006/03/17 10:24:27 alouis Exp $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.messaging.routing;

import java.util.List;

import org.ow2.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.component.context.ComponentContext;
import org.ow2.petals.jbi.messaging.exchange.MessageExchangeDecorator;

/**
 * Routes messages from the <code>DeliveryChannel</code> to their destinations.
 * Perform an address resolution, finding the endpoint(s) satisfying the
 * destination information, and select the appropriate transporter to convey the
 * messages. Resolutions are based on defined rules.
 * 
 * Routes messages pushed from the <code>Transporter</code> to the target
 * <code>DeliveryChannel</code>
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author rnaudin - EBM WebSourcing
 * @author chamerling - EBM WebSourcing
 */
public interface RouterService {
    
    /**
     * The message exchange property to set the TTL of the exchange.
     */
    static final String PROPERTY_ROUTER_TIMETOLIVE = "org.ow2.petals.router.timetolive";

    /**
     * The message exchange properties to set the NOACK feature.
     */
    static final String PROPERTY_ROUTER_CONSUMER_NOACK = "org.ow2.petals.router.consumer.noack";

    static final String PROPERTY_ROUTER_PROVIDER_NOACK = "org.ow2.petals.router.provider.noack";

    /**
     * The message exchange property name to set the provider link type to
     * resolve. See JBI specification section 5.4.3.3 for more details.
     */
    static final String PROPERTY_ROUTER_PROVIDER_LINKTYPE = "org.ow2.petals.router.provider.linktype";

    static final String DEFAULT_PLATFORM_STRATEGY = "highest,3,2,1";

    static final String DEFAULT_STANDALONE_STRATEGY = "standalone";

    /**
     * Constant used to specify the property name to set the router policy. Used
     * only in plaform distribution. The default policy is fast.
     */
    static final String PROPERTY_ROUTER_QOS = "org.ow2.petals.router.qos";

    /**
     * Constant used to specify the property name to set the router send
     * attempt. Used only in plaform distribution. The default attempt is 2.
     */
    static final String PROPERTY_ROUTER_SEND_ATTEMPT = "org.ow2.petals.router.send.attempt";

    /**
     * Constant used to specify the property name to set the delay between
     * router send attempts. Used only in plaform distribution. The default
     * attempt is 1 second.
     */
    static final String PROPERTY_ROUTER_SEND_DELAY = "org.ow2.petals.router.send.delay";

    /**
     * Constant used to specify the secure policy. It assures the security of
     * the exchange by encrypting the exchange sent
     */
    static final String SECURE_POLICY = "secure";

    /**
     * Constant used to specify the reliable policy. It assures the persistence
     * of the exchange. Message are never lost, but there is effort on the
     * duration of the exchange
     */
    static final String RELIABLE_POLICY = "reliable";

    /**
     * Constant used to specify the fast policy. It assures the fastest exchange
     * processing. If there is a problem during communication, the message can
     * be lost.
     */
    static final String FAST_POLICY = "fast";

    /**
     * The default QOS policy.
     */
    static final String DEFAULT_POLICY = FAST_POLICY;

    /**
     * Send the message into the NMR.
     * 
     * @param componentContext
     *            the source component context
     * @param exchangeDecorator
     *            the {@link MessageExchangeDecorator}
     * @throws RoutingException
     *             impossible to send the message
     */
    void send(ComponentContext componentContext, MessageExchangeDecorator exchangeDecorator)
            throws RoutingException;

    /**
     * Send the message synchronously into the NMR.
     * 
     * @param componentContext
     *            the source component context
     * @param exchangeDecorator
     *            the {@link MessageExchangeDecorator}
     * @param timeOut
     *            timeout of the message (in ms). 0 for no timeout
     * 
     * @throws RoutingException
     *             impossible to process a synchronous send
     */
    void sendSync(ComponentContext componentContext, MessageExchangeDecorator exchangeDecorator,
            long timeout) throws RoutingException;

    /**
     * Ask to receive a message from MNR and return it.
     * 
     * @param componentContext
     *            The source component context <b>MUST</b> be non null
     * @param timeoutMS
     *            parameter which represents
     *            <ul>
     *            <li>The time to wait if greater than 0</li>
     *            <li>A non blocking call if equal to 0</li>
     *            <li>Infinite Blocking call if less than 0</li>
     *            </ul>
     * 
     * @return The message exchange decorator retrieved from the
     *         <code>Transporter</code>
     * 
     * @throws RoutingException
     *             impossible to receive the message
     */
    MessageExchangeDecorator receive(ComponentContext componentContext, long timeoutMS)
            throws RoutingException;

    /**
     * Inform that a component is installing. The <code>Router</code> must
     * inform its <code>Transporters</code> about the component installation.
     * 
     * @param componentContext
     *            The component context
     * 
     * @throws RoutingException
     *             impossible to add the component
     */
    void addComponent(ComponentContext componentContext) throws RoutingException;

    /**
     * Inform that a component is uninstalling. The <code>Router</code> must
     * inform its <code>Transporters</code> about the component uninstallation.
     * 
     * @param componentContext
     *            The component context
     * 
     * @throws RoutingException
     *             impossible to remove the component
     */
    void removeComponent(ComponentContext componentContext) throws RoutingException;

    /**
     * Inform that a SA life-cycle has changed its state.
     * 
     * @param serviceUnitDescriptors
     *            the list of bound SU JBI descriptors
     */
    void modifiedSALifeCycle(List<ServiceUnitLifeCycle> serviceUnitLifes);

    /**
     * Prepare the stopping of the component. It means that all ongoing message
     * traffic must terminate during the method invocation and all new message
     * traffic is rejected.
     */
    void stopTraffic();
}
