/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ws;

import java.util.Iterator;
import java.util.List;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.management.deployment.AtomicDeploymentService;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.ws.api.DeploymentService;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.system.repository.artifact.Artifact;
import org.ow2.petals.system.repository.artifact.ArtifactRepositoryService;
import org.ow2.petals.tools.ws.KernelWebService;
import org.ow2.petals.util.LoggingUtil;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "webservice", signature = DeploymentService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class DeploymentServiceImpl implements DeploymentService, KernelWebService {

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Requires(name = "deployment", signature = AtomicDeploymentService.class)
    private AtomicDeploymentService deploymentService;

    @Requires(name = "artifactrepository", signature = ArtifactRepositoryService.class)
    private ArtifactRepositoryService artifactRepositoryService;

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.debug("Starting...");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * {@inheritDoc}
     */
    public boolean deploy(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : deploy with params saId = " + saId);
        }
        boolean result = true;

        List<Artifact> sas = this.artifactRepositoryService.getServiceAssemblies();
        Iterator<Artifact> iter = sas.iterator();
        boolean found = false;
        Artifact artifact = null;
        while (iter.hasNext() && !found) {
            Artifact a = iter.next();
            if ((a.getName() != null) && a.getName().equals(saId)) {
                artifact = a;
                found = true;
            }
        }

        if (artifact == null) {
            throw new PEtALSWebServiceException("Can not find artifact " + saId
                    + " in the repository");
        }

        try {
            result = this.deploymentService.deploy(artifact.getFile().toURL());
        } catch (Exception e) {
            String message = "Can not deploy the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public boolean shutdown(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : shutdown with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.shutdown(saId);
        } catch (PetalsException e) {
            String message = "Can not shutdown the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public boolean start(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : start with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.start(saId);
        } catch (PetalsException e) {
            String message = "Can not start the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public boolean stop(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : stop with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.stop(saId);
        } catch (PetalsException e) {
            String message = "Can not stop the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public boolean undeploy(String saId) throws PEtALSWebServiceException {
        if (this.log.isDebugEnabled()) {
            this.log.debug("Entering method : undeploy with params saId = " + saId);
        }
        boolean result = true;

        try {
            result = this.deploymentService.undeploy(saId);
        } catch (PetalsException e) {
            String message = "Can not undeploy the SA with ID " + saId;
            this.log.warning(message, e);
            throw new PEtALSWebServiceException(message + "(" + e.getMessage() + ")");
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

}
