/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 OW2 Consortium, http://www.ow2.org/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.ws.adapters;

import java.net.URI;
import java.net.URISyntaxException;

import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.kernel.configuration.DomainConfiguration;
import org.ow2.petals.kernel.configuration.JndiConfiguration;
import org.ow2.petals.kernel.configuration.SubDomainConfiguration;
import org.ow2.petals.kernel.configuration.ContainerConfiguration.RegistryMode;
import org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation;
import org.ow2.petals.kernel.ws.api.to.DomainInformation;
import org.ow2.petals.kernel.ws.api.to.JmxServiceInformation;
import org.ow2.petals.kernel.ws.api.to.JndiInformation;
import org.ow2.petals.kernel.ws.api.to.RegistryServiceInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation;
import org.ow2.petals.kernel.ws.api.to.TransportServiceInformation;
import org.ow2.petals.kernel.ws.api.to.WebServiceServiceInformation;
import org.ow2.petals.topology.generated.SubdomainMode;


public class TopologyInformationToTopologyConfiguration {

    /**
     * Adapts a ContainerInformation instance to a ContainerConfiguration one.
     *
     * @param containerInformation
     *      the ContainerInformation instance to be adapted
     * @return
     *      the corresponding ContainerConfiguration instance
     * @throws PetalsException
     *      if the RegistryMode is invalid
     */
    public static ContainerConfiguration containerInformationToContainerConfiguration(ContainerInformation containerInformation) throws PetalsException {
        ContainerConfiguration containerConfiguration = new ContainerConfiguration();

        if (containerInformation == null) {
            return null;
        }

        containerConfiguration.setName(containerInformation.getName());
        containerConfiguration.setDescription(containerInformation.getDescription());
        containerConfiguration.setHost(containerInformation.getHost());
        containerConfiguration.setUser(containerInformation.getUser());
        containerConfiguration.setPassword(containerInformation.getPassword());

        JmxServiceInformation jmxServiceInformation = containerInformation.getJmxService();
        if (jmxServiceInformation != null)  {
            containerConfiguration.setJmxRMIConnectorPort(jmxServiceInformation.getRmiPort());
        }

        TransportServiceInformation transportServiceInformation = containerInformation.getTransportService();
        if (transportServiceInformation != null) {
            containerConfiguration.setTCPPort(transportServiceInformation.getTcpPort());
        }

        RegistryServiceInformation registryServiceInformation = containerInformation.getRegistryService();
        if (registryServiceInformation != null) {
            containerConfiguration.setRegistryPort(registryServiceInformation.getPort());
        }

        WebServiceServiceInformation webServiceInformation = containerInformation.getWebserviceService();
        if (webServiceInformation != null) {
            containerConfiguration.setWebservicePort(webServiceInformation.getPort());
            containerConfiguration.setWebservicePrefix(webServiceInformation.getPrefix());
        }

        RegistryMode registryMode = null;
        switch (containerInformation.getNodeType()) {
            case MASTER :
                registryMode = RegistryMode.MASTER;
                break;
            case SLAVE :
                registryMode = RegistryMode.SLAVE;
                break;
            case  PEER :
                registryMode = RegistryMode.PEER;
                break;
            case STANDALONE :
                registryMode = RegistryMode.STANDALONE;
                break;
            default :
                throw new PetalsException("Unknown registry mode");
        }
        containerConfiguration.setRegistryMode(registryMode);

        return containerConfiguration;
    }

    /**
     * Adapts a SubdomainInformation to a SubDomainConfiguration one, without
     * taking care of the containers contained within the subdomain.
     *
     * @param subdomainInformation
     *      the SubdomainInformation instance to be adapted
     * @return
     *      the corresponding adapted SubDomainConfiguration instance
     * @throws PetalsException
     *      if the SubdomainMode is invalid
     */
    public static SubDomainConfiguration subdomainInformationToSubDomainConfiguration(SubdomainInformation subdomainInformation) throws PetalsException {
        SubDomainConfiguration subdomainConfiguration = new SubDomainConfiguration();

        if (subdomainInformation == null) {
            return null;
        }

        subdomainConfiguration.setName(subdomainInformation.getName());
        subdomainConfiguration.setDescription(subdomainInformation.getDescription());
        subdomainConfiguration.setNetworkTimeSynchronized(subdomainInformation.isNetworkTimeSynchronized());

        SubdomainMode subdomainMode;
        switch(subdomainInformation.getSubdomainMode()) {
            case MASTERSLAVE :
                subdomainMode = SubdomainMode.MASTER_SLAVE;
                break;
            case FLOODING :
                subdomainMode = SubdomainMode.FLOODING;
                break;
            case STANDALONE :
                subdomainMode = SubdomainMode.STANDALONE;
                break;
            default :
                throw new PetalsException("Unknown subdomain mode");
        }
        subdomainConfiguration.setMode(subdomainMode);


        return subdomainConfiguration;
    }

    /**
     * Adapts a DomainInformation instance to a DomainConfiguration one, without
     * taking care of the subdomains contained within the DomainInformation instance.
     *
     * @param domainInformation
     *      the DomainInformation instance to be adapted
     * @return
     *      the corresponding DomainConfiguration instance
     * @throws PetalsException
     *      if the domain mode is invalid
     */
    public static DomainConfiguration domainInformationToDomainInformation(DomainInformation domainInformation) throws PetalsException {
        DomainConfiguration domainConfiguration = new DomainConfiguration();

        if (domainInformation == null) {
            return null;
        }

        domainConfiguration.setName(domainInformation.getName());
        domainConfiguration.setDescription(domainInformation.getDescription());

        JndiInformation jndiInformation = domainInformation.getJndi();
        if (jndiInformation != null) {
            try {
                JndiConfiguration jndiConfiguration = new JndiConfiguration();
                domainConfiguration.setJndiConfiguration(jndiConfiguration);
                
                jndiConfiguration.setJndiProviderUrl(new URI(jndiInformation.getProviderUrl()));
                jndiConfiguration.setJndiFactory(jndiInformation.getFactory());

                jndiConfiguration.setJndiBatchSize(jndiInformation.getBatchSize());
                jndiConfiguration.setJndiPoolSize(jndiInformation.getPoolSize());
                
                jndiConfiguration.setJndiSecurityPrincipal(jndiInformation.getSecurityPrincipal());
                jndiConfiguration.setJndiSecurityCredentials(jndiInformation.getSecurityCredentials());
            } catch (URISyntaxException ex) {
                throw new PetalsException("No valid URI found for the JNDI provider URL", ex);
            }

        }

        DomainMode domainMode;
        switch(domainConfiguration.getMode()) {
            case STATIC :
                domainMode = DomainMode.STATIC;
                break;
            case STANDALONE :
                domainMode = DomainMode.STANDALONE;
                break;
            case DYNAMIC :
                domainMode = DomainMode.DYNAMIC;
                break;
            default :
                throw new PetalsException("Unknown domain mode");
        }
        domainConfiguration.setMode(domainMode);
        
        return domainConfiguration;
    }
}
